@@ -3,6 +3,7 @@
 # This file is part of gunicorn released under the MIT license. 
 # See the NOTICE for more information.
 
+import errno
 import logging
 import os
 import sys
@@ -18,20 +19,38 @@ class Application(object):
     An application interface for configuring and loading
     the various necessities for any given web framework.
     """
+    LOG_LEVELS = {
+        "critical": logging.CRITICAL,
+        "error": logging.ERROR,
+        "warning": logging.WARNING,
+        "info": logging.INFO,
+        "debug": logging.DEBUG
+    }
+    
     def __init__(self, usage=None):
         self.log = logging.getLogger(__name__)
-        self.cfg = Config(usage)
+        self.usage = usage
+        self.cfg = None
         self.callable = None
+        self.logger = None
+        self.load_config()
+  
+    def load_config(self):
+        # init configuration
+        self.cfg = Config(self.usage)
         
+        # parse console args
         parser = self.cfg.parser()
         opts, args = parser.parse_args()
+        
+        # optional settings from apps
         cfg = self.init(parser, opts, args)
         
         # Load up the any app specific configuration
         if cfg:
             for k, v in list(cfg.items()):
                 self.cfg.set(k.lower(), v)
-        
+                
         # Load up the config file if its found.
         if opts.config and os.path.exists(opts.config):
             cfg = {
@@ -64,14 +83,19 @@ class Application(object):
             if v is None:
                 continue
             self.cfg.set(k.lower(), v)
-            
-        self.configure_logging()
-    
+               
     def init(self, parser, opts, args):
         raise NotImplementedError
     
     def load(self):
         raise NotImplementedError
+
+    def reload(self):
+        self.load_config()
+        if self.cfg.spew:
+            debug.spew()
+        loglevel = self.LOG_LEVELS.get(self.cfg.loglevel.lower(), logging.INFO)
+        self.logger.setLevel(loglevel)
         
     def wsgi(self):
         if self.callable is None:
@@ -84,14 +108,22 @@ class Application(object):
         if self.cfg.daemon:
             util.daemonize()
         else:
-            os.setpgrp()
+            try:
+                os.setpgrp()
+            except OSError, e:
+                if e[0] == errno.EPERM:
+                    sys.stderr.write("Error: You should use "
+                        "daemon mode here.\n")
+                raise
+                    
+        self.configure_logging()
         Arbiter(self).run()
     
     def configure_logging(self):
         """\
         Set the log level and choose the destination for log output.
         """
-        logger = logging.getLogger('gunicorn')
+        self.logger = logging.getLogger('gunicorn')
 
         handlers = []
         if self.cfg.logfile != "-":
@@ -99,21 +131,13 @@ class Application(object):
         else:
             handlers.append(logging.StreamHandler())
 
-        levels = {
-            "critical": logging.CRITICAL,
-            "error": logging.ERROR,
-            "warning": logging.WARNING,
-            "info": logging.INFO,
-            "debug": logging.DEBUG
-        }
-
-        loglevel = levels.get(self.cfg.loglevel.lower(), logging.INFO)
-        logger.setLevel(loglevel)
+        loglevel = self.LOG_LEVELS.get(self.cfg.loglevel.lower(), logging.INFO)
+        self.logger.setLevel(loglevel)
         
         format = r"%(asctime)s [%(process)d] [%(levelname)s] %(message)s"
         datefmt = r"%Y-%m-%d %H:%M:%S"
         for h in handlers:
             h.setFormatter(logging.Formatter(format, datefmt))
-            logger.addHandler(h)
+            self.logger.addHandler(h)
 
 
