@@ -20,7 +20,22 @@ log = logging.getLogger(__name__)
 def create(req, sock, client, server, cfg):
     resp = Response(req, sock)
 
-    environ = {}
+    environ = {
+        "wsgi.input": req.body,
+        "wsgi.errors": sys.stderr,
+        "wsgi.version": (1, 0),
+        "wsgi.multithread": False,
+        "wsgi.multiprocess": (cfg.workers > 1),
+        "wsgi.run_once": False,
+        "gunicorn.socket": sock,
+        "SERVER_SOFTWARE": SERVER_VERSION,
+        "REQUEST_METHOD": req.method,
+        "QUERY_STRING": req.query,
+        "RAW_URI": req.uri,
+        "SERVER_PROTOCOL": "HTTP/%s" % ".".join(map(str, req.version)),
+        "CONTENT_TYPE": "",
+        "CONTENT_LENGTH": ""
+    }
     
     # authors should be aware that REMOTE_HOST and REMOTE_ADDR
     # may not qualify the remote addr:
@@ -29,33 +44,34 @@ def create(req, sock, client, server, cfg):
     forward = client
     url_scheme = "http"
     script_name = os.environ.get("SCRIPT_NAME", "")
-    content_type = ""
-    content_length = ""
+
     for hdr_name, hdr_value in req.headers:
-        name = hdr_name.lower()
-        if name == "expect":
+        if hdr_name == "EXPECT":
             # handle expect
             if hdr_value.lower() == "100-continue":
                 sock.send("HTTP/1.1 100 Continue\r\n\r\n")
-        elif name == "x-forwarded-for":
+        elif hdr_name == "X-FORWARDED-FOR":
             forward = hdr_value
-        elif name == "x-forwarded-protocol" and hdr_value.lower() == "ssl":
+        elif hdr_name == "X-FORWARDED-PROTOCOL" and hdr_value.lower() == "ssl":
             url_scheme = "https"
-        elif name == "x-forwarded-ssl" and hdr_value.lower() == "on":
+        elif hdr_name == "X-FORWARDED-SSL" and hdr_value.lower() == "on":
             url_scheme = "https"
-        elif name == "host":
+        elif hdr_name == "HOST":
             server = hdr_value
-        elif name == "script_name":
+        elif hdr_name == "SCRIPT_NAME":
             script_name = hdr_value
-        elif name == "content-type":
-            content_type = hdr_value
-        elif name == "content-length":
-            content_length = hdr_value
-        else:
+        elif hdr_name == "CONTENT-TYPE":
+            environ['CONTENT_TYPE'] = hdr_value
             continue
+        elif hdr_name == "CONTENT-LENGTH":
+            environ['CONTENT_LENGTH'] = hdr_value
+            continue
+        
+        key = 'HTTP_' + hdr_name.replace('-', '_')
+        environ[key] = hdr_value
 
-    wsgi_multiprocess = (cfg.workers > 1)
-
+    environ['wsgi.url_scheme'] = url_scheme
+        
 
     if isinstance(forward, basestring):
         # we only took the last one
@@ -63,10 +79,13 @@ def create(req, sock, client, server, cfg):
         if forward.find(",") >= 0:
             forward = forward.rsplit(",", 1)[1].strip()
         remote = forward.split(":")
-        if len(remote) == 1:
-            remote.append('')
+        if len(remote) < 2:
+            remote.append('80')
     else:
-        remote = forward
+        remote = forward 
+
+    environ['REMOTE_ADDR'] = remote[0]
+    environ['REMOTE_PORT'] = str(remote[1])
 
     if isinstance(server, basestring):
         server =  server.split(":")
@@ -77,40 +96,15 @@ def create(req, sock, client, server, cfg):
                 server.append("443")
             else:
                 server.append('')
+    environ['SERVER_NAME'] = server[0]
+    environ['SERVER_PORT'] = server[1]
 
     path_info = req.path
     if script_name:
         path_info = path_info.split(script_name, 1)[1]
+    environ['PATH_INFO'] = unquote(path_info)
+    environ['SCRIPT_NAME'] = script_name
 
-    environ = {
-        "wsgi.url_scheme": url_scheme,
-        "wsgi.input": req.body,
-        "wsgi.errors": sys.stderr,
-        "wsgi.version": (1, 0),
-        "wsgi.multithread": False,
-        "wsgi.multiprocess": wsgi_multiprocess,
-        "wsgi.run_once": False,
-        "gunicorn.socket": sock,
-        "SCRIPT_NAME": script_name,
-        "SERVER_SOFTWARE": SERVER_VERSION,
-        "REQUEST_METHOD": req.method,
-        "PATH_INFO": unquote(path_info),
-        "QUERY_STRING": req.query,
-        "RAW_URI": req.uri,
-        "CONTENT_TYPE": content_type,
-        "CONTENT_LENGTH": content_length,
-        "REMOTE_ADDR": remote[0],
-        "REMOTE_PORT": str(remote[1]),
-        "SERVER_NAME": server[0],
-        "SERVER_PORT": str(server[1]),
-        "SERVER_PROTOCOL": "HTTP/%s" % ".".join(map(str, req.version))
-    }
-
-    for key, value in req.headers:
-        key = 'HTTP_' + key.upper().replace('-', '_')
-        if key not in ('HTTP_CONTENT_TYPE', 'HTTP_CONTENT_LENGTH'):
-            environ[key] = value
-           
     return resp, environ
 
 class Response(object):
@@ -185,4 +179,3 @@ class Response(object):
             self.send_headers()
         if self.chunked:
             util.write_chunk(self.sock, "")
-
