@@ -33,9 +33,11 @@ class AsyncWorker(base.Worker):
                     req = None
                     with self.timeout_ctx():
                         req = parser.next()
+
                     if not req:
                         break
                     self.handle_request(req, client, addr)
+                
             except StopIteration:
                 pass
         except socket.error, e:
@@ -43,7 +45,7 @@ class AsyncWorker(base.Worker):
                 self.log.exception("Socket error processing request.")
             else:
                 if e[0] == errno.ECONNRESET:
-                    self.log.warn("Ignoring connection reset")
+                    self.log.debug("Ignoring connection reset")
                 else:
                     self.log.debug("Ignoring EPIPE")
         except Exception, e:
@@ -75,6 +77,8 @@ class AsyncWorker(base.Worker):
             if req.should_close():
                 raise StopIteration()
             self.cfg.post_request(self, req)
+        except StopIteration:
+            raise
         except Exception, e:
             #Only send back traceback in HTTP in debug mode.
             if not self.debug:
