@@ -10,8 +10,8 @@ import os
 import gevent
 from gevent import monkey
 monkey.noisy = False
-from gevent import greenlet
 from gevent.pool import Pool
+from gevent.server import StreamServer
 from gevent import pywsgi, wsgi
 
 import gunicorn
@@ -30,8 +30,25 @@ BASE_WSGI_ENV = {
     'wsgi.run_once': False
 }
 
+
+class GGeventServer(StreamServer):
+    def __init__(self, listener, handle, spawn='default', worker=None):
+        StreamServer.__init__(self, listener, spawn=spawn)
+        self.handle_func = handle
+        self.worker = worker
+
+    def stop(self):
+        super(GGeventServer, self).stop()
+        self.worker.alive = False
+
+    def handle(self, sock, addr):
+        self.handle_func(sock, addr)
+
 class GeventWorker(AsyncWorker):
-        
+
+    def __init__(self, *args, **kwargs):
+        super(GeventWorker, self).__init__(*args, **kwargs)
+
     @classmethod  
     def setup(cls):
         from gevent import monkey
@@ -39,26 +56,29 @@ class GeventWorker(AsyncWorker):
         
     def timeout_ctx(self):
         return gevent.Timeout(self.cfg.keepalive, False)
-        
+
     def run(self):
         self.socket.setblocking(1)
 
         pool = Pool(self.worker_connections)
-        acceptor = gevent.spawn(self.acceptor, pool)
-        
+        server = GGeventServer(self.socket, self.handle, spawn=pool,
+                worker=self)
+
+        server.start()
+
         try:
             while self.alive:
                 self.notify()
-            
                 if self.ppid != os.getppid():
                     self.log.info("Parent changed, shutting down: %s" % self)
-                    gevent.kill(acceptor)
                     break
-                gevent.sleep(0.1)
-            pool.join(timeout=self.timeout)
-        except KeyboardInterrupt:
+                gevent.sleep(self.timeout)
+        except:
             pass
 
+        with gevent.Timeout(self.timeout, False):
+            gevent.spawn(server.stop).join()
+
     def init_process(self):
         #gevent doesn't reinitialize dns for us after forking
         #here's the workaround
@@ -66,31 +86,6 @@ class GeventWorker(AsyncWorker):
         gevent.core.dns_init()
         super(GeventWorker, self).init_process()
 
-    def acceptor(self, pool):
-        gevent.getcurrent()
-        while self.alive:
-            try:
-                conn, addr = self.socket.accept()
-                gt = pool.spawn(self.handle, conn, addr)
-                gt._conn = conn
-                gt.link(self.cleanup)
-                conn, addr, gt = None, None, None
-            except greenlet.GreenletExit:
-                return
-            except:
-                self.log.exception("Unexpected error in acceptor. Sepuku.")
-                os._exit(4)
-          
-    def cleanup(self, gt):
-        try:
-            gt.join()
-        except greenlet.GreenletExit:
-            pass
-        except Exception:
-            self.log.exception("Unhandled exception in worker.")
-        finally:
-            gt._conn.close()
-
 class GeventBaseWorker(Worker):
     """\
     This base class is used for the two variants of workers that use
@@ -128,10 +123,11 @@ class GeventBaseWorker(Worker):
                     server.stop()
                     break
                 gevent.sleep(0.1) 
-            self.pool.join(timeout=self.timeout)
         except KeyboardInterrupt:
             pass
-
+      
+        with gevent.Timeout(self.timeout, False):
+            gevent.spawn(server.stop).join()
 
 class WSGIHandler(wsgi.WSGIHandler):
     def log_request(self, *args):
